@@ -5,7 +5,8 @@ import os
 import pathlib
 import subprocess
 from contextlib import contextmanager
-from typing import Any, Iterator, List, Type
+from shutil import rmtree
+from typing import Any, Iterator, List, Type, Union
 
 import pytest
 from _pytest.monkeypatch import MonkeyPatch
@@ -19,7 +20,7 @@ from ansible_compat.errors import (
     AnsibleCompatError,
     InvalidPrerequisiteError,
 )
-from ansible_compat.runtime import CompletedProcess, Runtime, _update_env
+from ansible_compat.runtime import CompletedProcess, Runtime
 
 
 def test_runtime_version(runtime: Runtime) -> None:
@@ -128,11 +129,7 @@ def test_runtime_install_role(
     caplog.set_level(logging.INFO)
     project_dir = os.path.join(os.path.dirname(__file__), "roles", folder)
     runtime = Runtime(isolated=isolated, project_dir=project_dir)
-    runtime.prepare_environment()
-    assert (
-        "symlink to current repository in order to enable Ansible to find the role"
-        in caplog.text
-    )
+    runtime.prepare_environment(install_local=True)
     # check that role appears as installed now
     result = runtime.exec(["ansible-galaxy", "list"])
     assert result.returncode == 0, result
@@ -256,27 +253,30 @@ def test__update_env_no_old_value_no_default_no_value(monkeypatch: MonkeyPatch)
     """Make sure empty value does not touch environment."""
     monkeypatch.delenv("DUMMY_VAR", raising=False)
 
-    _update_env("DUMMY_VAR", [])
+    runtime = Runtime()
+    runtime._update_env("DUMMY_VAR", [])
 
-    assert "DUMMY_VAR" not in os.environ
+    assert "DUMMY_VAR" not in runtime.environ
 
 
 def test__update_env_no_old_value_no_value(monkeypatch: MonkeyPatch) -> None:
     """Make sure empty value does not touch environment."""
     monkeypatch.delenv("DUMMY_VAR", raising=False)
 
-    _update_env("DUMMY_VAR", [], "a:b")
+    runtime = Runtime()
+    runtime._update_env("DUMMY_VAR", [], "a:b")
 
-    assert "DUMMY_VAR" not in os.environ
+    assert "DUMMY_VAR" not in runtime.environ
 
 
 def test__update_env_no_default_no_value(monkeypatch: MonkeyPatch) -> None:
     """Make sure empty value does not touch environment."""
     monkeypatch.setenv("DUMMY_VAR", "a:b")
 
-    _update_env("DUMMY_VAR", [])
+    runtime = Runtime()
+    runtime._update_env("DUMMY_VAR", [])
 
-    assert os.environ["DUMMY_VAR"] == "a:b"
+    assert runtime.environ["DUMMY_VAR"] == "a:b"
 
 
 @pytest.mark.parametrize(
@@ -293,9 +293,10 @@ def test__update_env_no_old_value_no_default(
     """Values are concatenated using : as the separator."""
     monkeypatch.delenv("DUMMY_VAR", raising=False)
 
-    _update_env("DUMMY_VAR", value)
+    runtime = Runtime()
+    runtime._update_env("DUMMY_VAR", value)
 
-    assert os.environ["DUMMY_VAR"] == result
+    assert runtime.environ["DUMMY_VAR"] == result
 
 
 @pytest.mark.parametrize(
@@ -311,9 +312,10 @@ def test__update_env_no_old_value(
     """Values are appended to default value."""
     monkeypatch.delenv("DUMMY_VAR", raising=False)
 
-    _update_env("DUMMY_VAR", value, default)
+    runtime = Runtime()
+    runtime._update_env("DUMMY_VAR", value, default)
 
-    assert os.environ["DUMMY_VAR"] == result
+    assert runtime.environ["DUMMY_VAR"] == result
 
 
 @pytest.mark.parametrize(
@@ -329,9 +331,10 @@ def test__update_env_no_default(
     """Values are appended to preexisting value."""
     monkeypatch.setenv("DUMMY_VAR", old_value)
 
-    _update_env("DUMMY_VAR", value)
+    runtime = Runtime()
+    runtime._update_env("DUMMY_VAR", value)
 
-    assert os.environ["DUMMY_VAR"] == result
+    assert runtime.environ["DUMMY_VAR"] == result
 
 
 @pytest.mark.parametrize(
@@ -353,9 +356,10 @@ def test__update_env(
     """Defaults are ignored when preexisting value is present."""
     monkeypatch.setenv("DUMMY_VAR", old_value)
 
-    _update_env("DUMMY_VAR", value)
+    runtime = Runtime()
+    runtime._update_env("DUMMY_VAR", value)
 
-    assert os.environ["DUMMY_VAR"] == result
+    assert runtime.environ["DUMMY_VAR"] == result
 
 
 def test_require_collection_wrong_version(runtime: Runtime) -> None:
@@ -550,3 +554,96 @@ def test_runtime_env_ansible_library(monkeypatch: MonkeyPatch) -> None:
     runtime = Runtime()
     runtime.prepare_environment()
     assert path_name in runtime.config.default_module_path
+
+
+@pytest.mark.parametrize(
+    ("lower", "upper", "expected"),
+    (
+        ("1.0", "9999.0", True),
+        (None, "9999.0", True),
+        ("1.0", None, True),
+        ("9999.0", None, False),
+        (None, "1.0", False),
+    ),
+    ids=("1", "2", "3", "4", "5"),
+)
+def test_runtime_version_in_range(
+    lower: Union[str, None], upper: Union[str, None], expected: bool
+) -> None:
+    """Validate functioning of version_in_range."""
+    runtime = Runtime()
+    assert runtime.version_in_range(lower=lower, upper=upper) is expected
+
+
+@pytest.mark.parametrize(
+    ("path", "scenario"),
+    (
+        ("test/collections/acme.goodies", "default"),
+        ("test/collections/acme.goodies/roles/baz", "deep_scenario"),
+    ),
+    ids=("normal", "deep"),
+)
+def test_install_collection_from_disk(path: str, scenario: str) -> None:
+    """Tests ability to install a local collection."""
+    # ensure we do not have acme.google installed in user directory as it may
+    # produce false positives
+    rmtree(
+        os.path.expanduser("~/.ansible/collections/ansible_collections/acme/goodies"),
+        ignore_errors=True,
+    )
+    with remember_cwd(path):
+        runtime = Runtime(isolated=True)
+        # this should call install_collection_from_disk(".")
+        runtime.prepare_environment(install_local=True)
+        # that molecule converge playbook can be used without molecule and
+        # should validate that the installed collection is available.
+        result = runtime.exec(["ansible-playbook", f"molecule/{scenario}/converge.yml"])
+        assert result.returncode == 0, result.stdout
+        runtime.clean()
+
+
+def test_install_collection_from_disk_fail() -> None:
+    """Tests that we fail to install a broken collection."""
+    with remember_cwd("test/collections/acme.broken"):
+        runtime = Runtime(isolated=True)
+        exception: Type[Exception]
+        if runtime.version_in_range(upper="2.11"):
+            exception = AnsibleCommandError
+            msg = "Got 1 exit code while running: ansible-galaxy collection build"
+        else:
+            exception = InvalidPrerequisiteError
+            msg = "is missing the following mandatory"
+        with pytest.raises(exception, match=msg):
+            # this should call install_collection_from_disk(".")
+            runtime.prepare_environment(install_local=True)
+
+
+def test_runtime_run(runtime: Runtime) -> None:
+    """Check if tee and non tee mode return same kind of results."""
+    result1 = runtime.exec(["seq", "10"])
+    result2 = runtime.exec(["seq", "10"], tee=True)
+    assert result1.returncode == result2.returncode
+    assert result1.stderr == result2.stderr
+    assert result1.stdout == result2.stdout
+
+
+def test_runtime_exec_cwd(runtime: Runtime) -> None:
+    """Check if passing cwd works as expected."""
+    cwd = "/"
+    result1 = runtime.exec(["pwd"], cwd=cwd)
+    result2 = runtime.exec(["pwd"])
+    assert result1.stdout.rstrip() == cwd
+    assert result1.stdout != result2.stdout
+
+
+def test_runtime_exec_env(runtime: Runtime) -> None:
+    """Check if passing env works."""
+    result = runtime.exec(["printenv", "FOO"])
+    assert not result.stdout
+
+    result = runtime.exec(["printenv", "FOO"], env={"FOO": "bar"})
+    assert result.stdout.rstrip() == "bar"
+
+    runtime.environ["FOO"] = "bar"
+    result = runtime.exec(["printenv", "FOO"])
+    assert result.stdout.rstrip() == "bar"
